require 'spec_helper'

describe Geocodable do
  describe '#api_key=' do
    it 'sets the API key' do
      Geocodable.api_key = 'abc123'
      expect(Geocodable.api_key).to eq('abc123')
    end
  end

  describe '#geocode' do
    before { Geocodable.api_key = 'abc' }

    it 'returns results' do
      stub_api_request(body: '{"found": 1}')
      resp = Geocodable.geocode('London')
      expect(resp[:found]).to eq(1)
    end

    it 'raises InvalidRequestError with missing query param' do
      stub_api_request(params: { query: '' },
                       status: 400, 
                       body: '{"errors": ["Query parameter missing"]}')
      expect { 
        Geocodable.geocode('')
      }.to raise_error(Geocodable::InvalidRequestError) { |e|
        expect(e.message).to eq('Query parameter missing')
        expect(e.http_status).to eq(400)
        expect(e.http_body).to eq(errors: ['Query parameter missing'])
      }
    end

    it 'raises AuthenticationError with invalid API key' do
      stub_api_request(headers: {},
                       status: 401, 
                       body: '{"errors": ["Unauthorized API key"]}')
      Geocodable.api_key = nil
      expect { 
        Geocodable.geocode('London')
      }.to raise_error(Geocodable::AuthenticationError) { |e|
        expect(e.message).to eq('Unauthorized API key')
        expect(e.http_status).to eq(401)
        expect(e.http_body).to eq(errors: ['Unauthorized API key'])
      }
    end

    it 'raises AccessDisabledError when api_key disabled' do
      stub_api_request(status: 403, 
                       body: '{"errors": ["API key disabled by owner"]}')
      expect { 
        Geocodable.geocode('London')
      }.to raise_error(Geocodable::AccessDisabledError) { |e|
        expect(e.message).to eq('API key disabled by owner')
        expect(e.http_status).to eq(403)
        expect(e.http_body).to eq(errors: ['API key disabled by owner'])
      }
    end

    it 'raises OverRequestLimitError when api_key disabled' do
      stub_api_request(status: 429, body: '{"errors": ["Over request limit"]}')
      expect { 
        Geocodable.geocode('London')
      }.to raise_error(Geocodable::OverRequestLimitError) { |e|
        expect(e.message).to eq('Over request limit')
        expect(e.http_status).to eq(429)
        expect(e.http_body).to eq(errors: ['Over request limit'])
      }
    end

    it 'raises APIError when 500 server error' do
      stub_api_request(status: 500, 
                       body: '{"status":"500", "error":"Internal server error"}')
      expect { 
        Geocodable.geocode('London')
      }.to raise_error(Geocodable::APIError) { |e|
        expect(e.message).to eq('Internal server error')
        expect(e.http_status).to eq(500)
        expect(e.http_body).to eq(error: 'Internal server error', status: '500')
      }
    end
  end
end


def stub_api_request(args)
  args = { 
    headers: { authorization: 'Bearer abc' }, 
    params: { query: 'London' }, 
    status: 200, 
    body: ''
  }.update(args)
  query = URI.encode_www_form(args[:params])
  args[:headers].update(
    'Host' => 'api.geocodable.io:443', 
    'User-Agent' => "geocodable-ruby/#{Geocodable::VERSION}"
  )
  stub_request(:get, "https://api.geocodable.io/v1/geocode.json?#{query}").
    with(headers: args[:headers]).
    to_return(
      body: args[:body], 
      status: args[:status], 
      headers: { 'Content-Type' => 'application/json' }
    )
end
